#!/usr/bin/perl
#
# $Id$
#
# (C) IBCL Informatik-Buero e.U. 2010
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

use strict;

use Dpkg::Version;


require "/usr/lib/pba/pba-functions.pl";


sub prepareGetopts() {

$main::helpMessage = '
 Syntax:
  '.$main::scriptName.' CommandOption [Options]
  
 CommandOptions are:
  -A          Automatic mode, import eventual incoming .dsc files.
  -L          List dcg distro-category-generation. Add -q for looping over.
  -R d-c-g    Refresh release of given distro-category-generation.
  --help      Prints this help text and exits.
  --version   Prints the version info and exits.

 Common options are:
  -c configdir     Read configdir/* rather than /etc/pba/*
  -d               Debug mode, logs everything.
  -q               Quiet mode, supresses informative messages.
  -v               Verbose verboser, logs more.
  -x               eXpunge obsolete packages 
 
 For some useful shell loops take a look at ReadMe.txt, usually at
 /usr/share/doc/pba-common/ReadMe.txt
';

  $main::optionList = "ALR:c:dlqvx";
  $main::role = "archiver";

}


sub initArchiveFunctions() { 

  $main::configParams{"ArchiveReportCc"} = "";
  $main::configParams{"GPGHome"} = "";
  $main::configParams{"GPGKeyId"} = "";
  $main::configParams{"GPGOptions"} = "";
  $main::configParams{"IncomingDirs"} = "";
  $main::configParams{"TargetRepository"} = "";

}


sub checkArchiverConfig() {

  # check config
  $main::incomingDirs = simplifyList($main::configParams{"IncomingDirs"});
  ($main::incomingDirs eq "") && cleanupAndDie("FATAL: No IncomingDirs specified in ".$main::configFile."\n");

  $main::gpgHome = $main::configParams{"GPGHome"};
  $main::gpgKeyId = $main::configParams{"GPGKeyId"};
  (length($main::configParams{"GPGOptions"}) > 0) && cleanupAndDie("FATAL: Obsolete option GPGOptions found in ".$main::configFile.", please migrate to GPGHome, GPGKeyId.\n");

  $main::targetRepository = $main::configParams{"TargetRepository"};
  ($main::targetRepository eq "") && cleanupAndDie("FATAL: No TargetRepository specified in ".$main::configFile."\n");

  my $repositoryRoot = $main::repositoryRoots{$main::targetRepository};
  ($repositoryRoot eq "") && cleanupAndDie("FATAL: Don't know where my repository root is.\n");
  (not -d $repositoryRoot) && cleanupAndDie("FATAL: Repository root directory does not exist: ".$repositoryRoot."\n");
  ($main::verbosity > 2) && print "Repository root looks good: ".$repositoryRoot."\n";

  foreach my $repository (keys(%main::repositoryRoots)) {
     ($main::repositoryRoots{$repository} eq "") && cleanupAndDie("FATAL: No RepositoryRoot specified for repository ".$repository."\n");
  }

  my $commandOptions = 0;
  if ($main::opts{"A"} ne "") {
    $main::archiveOperation = "auto-import";
    $commandOptions++;
  }
  if ($main::opts{"L"} ne "") {
    $main::archiveOperation = "list-archives";
    $commandOptions++;
  }
  if ($main::opts{"R"} ne "") {
    $main::archiveOperation = "refresh-archive";
    $main::targetDCGArchive = $main::opts{"R"};
    $commandOptions++;
  }
  #$main::forcedRefreshDCG = $main::opts{"r"};

  # check command and options
  if ($commandOptions == 0) {
    print "FATAL: No command specified!\n";
    print $main::helpMessage;
    exit 1;
  }
  if ($commandOptions > 1) {
    cleanupAndDie("FATAL: Conflicting Commands (only one of -A, -L, -R, allowed)!\n");
  }
  if ($main::archiveOperation eq "refresh-archive") {
    ($main::targetDCGArchive eq "") && cleanupAndDie("FATAL: No archive distribution-category-generation given to refresh.\n");
    my ($dcgDistribution, $dcgCategory, $dcgGeneration) = split /-/, $main::targetDCGArchive, 3;
    (defined $main::distributionLongNames{$dcgDistribution}) || cleanupAndDie("FATAL: Unknown distribution ".$dcgDistribution."\n");
  }
  if ($main::opts{"x"} eq "") {
    $main::eXpungeOldDebs = 0;
  } else {
    $main::eXpungeOldDebs = 1;
  }

  # debug infos
  if ($main::verbosity > 2) {
    print "Incoming directories: ".$main::incomingDirs."\n";
    print "Target Repository: ".$main::targetRepository."\n";
    print "GPG Home: '".$main::gpgHome."'\n";
    print "GPG KeyId: '".$main::gpgKeyId."'\n";
  }


}


sub scanForIncomingTargzFiles() {
  @main::packageFilesToImport = ();
  my @dirsToScan = split / +/, $main::incomingDirs;
  foreach my $dirToScan (@dirsToScan) {
    ($main::verbosity > 2) && print "Scanning incoming directory ".$dirToScan."\n";
    chdir $dirToScan || cleanupAndDie("FATAL: Cannot chdir to ".$dirToScan."\n");
    my @files = glob("*.debs.tar.gz");
    foreach my $file (@files) {
      next if (! -f $file); # skip dirs
      ($main::verbosity > 2) && print "Found tarball: ".$file."\n";
      push @main::packageFilesToImport, $dirToScan."/".$file;
    }
  }
}


sub importDebFile( $ ) {
  # $1 = fullpath of file to import
  # returns distribution-category-generation
  my $packageFile = shift;
  ($packageFile eq "") && cleanupAndDie("FATAL: importDebFile(\$) called w/o argument.\n");
  (-f $packageFile) || cleanupAndDie("FATAL: importDebFile(".$packageFile."): No such file.\n");

  ($main::verbosity > 2) && print "Analysing package ".$packageFile."\n";
  # analyse package, check if we are responsible
  my $packageCategory = "";
  my $packageDistribution = "";
  my $packageGeneration = "";
  my $packageRepositories = "";
  my $dpkgCommand = "dpkg-deb -I ".$packageFile;
  open(DPKG, $dpkgCommand."|") || cleanupAndDie("FATAL: Cannot read output of ".$dpkgCommand."\n");
  while ( my $curLine = <DPKG> ) {
    chomp $curLine;
    my ( $pn, $pv ) = split /: +/, $curLine, 2;
    #next if ($pn != /^PBA-[a-z]/i);
    if ($pn =~ /^ PBA-Category$/i) {
      $packageCategory = $pv;
      next;
    }
    if ($pn =~ /^ PBA-Distribution$/i) {
      $packageDistribution = $pv;
      next;
    }
    if ($pn =~ /^ PBA-Generation$/i) {
      $packageGeneration = $pv;
      next;
    }
    if ($pn =~ /^ PBA-Repositories$/i) {
      $packageRepositories = $pv;
      next;
    }
  }
  close DPKG;
  ($packageCategory eq "") && cleanupAndDie("FATAL: No PBA-Category specified in ".$packageFile."\n");
  if ($packageDistribution eq "") {
    foreach my $supportedDistro (sort(keys(%main::distributionLongNames))) {
      my $pattern = "_".$supportedDistro."_[a-z0-9]+\\.deb".'$';
      if ($packageFile =~ /$pattern/) {
        $packageDistribution = $supportedDistro;
      }
    } 
    ($packageDistribution eq "") && cleanupAndDie("FATAL: No PBA-Distribution specified in ".$packageFile."\n");
  }
  ($packageGeneration eq "") && cleanupAndDie("FATAL: No PBA-Generation specified in ".$packageFile."\n");
  (defined $main::distributionLongNames{$packageDistribution}) || cleanupAndDie("FATAL: Unknown distribution ".$packageDistribution." in ".$packageFile."\n");
  my $packageFamily = $main::distributionFamilies{$packageDistribution}; 
  if ($packageRepositories ne "") {
    my $packageAllowsRepository = 0;
    foreach my $packageRepository (split /,? +/, $packageRepositories) {
      if ($main::targetRepository eq $packageRepository) 
      { $packageAllowsRepository += 1; }
    }
    ($packageAllowsRepository == 0) && cleanupAndDie("FATAL: Package does not allow my target repository ".$main::targetRepository.": ".$packageFile."\n");
  }

  # move package to correct place 
  
  my $packageDCG = $packageDistribution."-".$packageCategory."-".$packageGeneration; 
  my $command; 
  my $rc;
  
  my $repositoryRoot = $main::repositoryRoots{$main::targetRepository};
  my $packagePoolDir = $repositoryRoot."/".$main::distributionFamilies{$packageDistribution}."/pool";
  my $packageDir = $packagePoolDir."/".$packageDCG;
  ($main::verbosity > 2) && print "Package directory is ".$packageDir.".\n";
  checkDirectory($packageDir);
  
  chmod 0755, $packageFile || cleanupAndDie("FATAL: Cannot chmod ".$packageFile."\n");
  executeOrDie("/bin/mv ".$packageFile." ".$packageDir."/");

  return $packageDCG;

}


sub refreshDCGArchive ( $ ) { 
  # $1 = distribution-category-generation
  my $archiveDCG = shift;
  ($archiveDCG eq "") && cleanupAndDie("FATAL: refreshDCGArchive(\$) called w/o argument.\n");

  my ($dcgDistribution, $dcgCategory, $dcgGeneration) = split /-/, $archiveDCG, 3;
  (defined $main::distributionLongNames{$dcgDistribution}) || cleanupAndDie("FATAL: Unknown distribution ".$dcgDistribution."\n");
  my $repositoryRoot = $main::repositoryRoots{$main::targetRepository};
  my $dcgPoolDir = $repositoryRoot."/".$main::distributionFamilies{$dcgDistribution}."/pool";
  (-d $dcgPoolDir."/".$archiveDCG) || cleanupAndDie("FATAL: Unknown generation ".$dcgGeneration."\n");

  ($main::verbosity > 2) && print "Refreshing Release info for ".$archiveDCG."\n";

  my $dcgFamilyBaseDir = $repositoryRoot."/".$main::distributionFamilies{$dcgDistribution};
  my $dcgDistsDCGDir = $dcgFamilyBaseDir."/dists/".$archiveDCG;
  #chdir $dcgBaseDir || cleanupAndDie("FATAL: Cannot chdir to ".$dcgBaseDir."\n");
  my $dcgPoolDCGDir = $dcgFamilyBaseDir."/pool/".$archiveDCG;

  # detect and evtl. /bin/rm old .deb files
  my %recentDebsByNameArch = (); # holds most recent deb of any name-arch combination
  my %versionsByDeb = ();
  my @obsoleteDebs = ();
  chdir $dcgPoolDCGDir || cleanupAndDie("FATAL: Cannot chdir to ".$dcgPoolDCGDir."\n");
  my @debFiles = glob("*.deb");
  foreach my $debFile (@debFiles) {
    next if (! -f $debFile); # skip dirs
    my $packageName = "";
    my $packageArchitecture = "";
    my $packageVersion = "";
    my $dpkgCommand = "dpkg-deb -I ".$debFile;
    open(DPKG, $dpkgCommand."|") || cleanupAndDie("FATAL: Cannot read output of ".$dpkgCommand."\n");
    while ( my $curLine = <DPKG> ) {
      chomp $curLine;
      my ( $pn, $pv ) = split /: +/, $curLine, 2;
      if ($pn =~ /^ Architecture$/i) {
        $packageArchitecture = $pv;
        next;
      }
      if ($pn =~ /^ Package$/i) {
        $packageName = $pv;
        next;
      }
      if ($pn =~ /^ Version$/i) {
        $packageVersion = $pv;
        next;
      }
    }
    close DPKG;
    ($packageArchitecture eq "") && cleanupAndDie("FATAL: Cannot detect architecture of ".pwd()."/".$debFile."\n");
    ($packageName eq "") && cleanupAndDie("FATAL: Cannot detect package name of ".pwd()."/".$debFile."\n");
    ($packageVersion eq "") && cleanupAndDie("FATAL: Cannot detect version of ".pwd()."/".$debFile."\n");
    $versionsByDeb{$debFile} = $packageVersion;
    my $packageNameArch = $packageName."-".$packageArchitecture;
    if ($recentDebsByNameArch{$packageNameArch} eq "") {
      # new nameArch
      $recentDebsByNameArch{$packageNameArch} = $debFile;
    } else {
      my $otherVersion = $versionsByDeb{$recentDebsByNameArch{$packageNameArch}};
      if (version_compare($packageVersion, $otherVersion) > 0) { 
        # current is newer
        push @obsoleteDebs, $recentDebsByNameArch{$packageNameArch};
        $recentDebsByNameArch{$packageNameArch} = $debFile;
      } else {
        # other is newer
        push @obsoleteDebs, $debFile;
      }
    }
  }
  if (scalar(@obsoleteDebs) > 0) {
    if ($main::eXpungeOldDebs) {
      print "Removing obsolete .deb files in ".$dcgPoolDCGDir."\n";
      foreach my $obsoleteDeb (@obsoleteDebs) {
        unlink $obsoleteDeb || cleanupAndDie("FATAL: Cannot delete ".pwd()."/".$obsoleteDeb."\n");
      }
    }
  } else {
    print "No obsolete .deb files to remove.\n";
  }

  my @dcgArchitectures = split /,? +/, $main::distributionArchitectures{$dcgDistribution};
  
  checkDirectory($dcgDistsDCGDir."/main/i18n");

  # dists/$dcg/binary-$arch
  foreach my $dcgArchitecture (@dcgArchitectures) {

    chdir $dcgFamilyBaseDir || cleanupAndDie("FATAL: Cannot chdir to ".$dcgFamilyBaseDir."\n");
    my $binArchDir = "dists/".$archiveDCG."/main/binary-".$dcgArchitecture;
    checkDirectory($dcgFamilyBaseDir."/".$binArchDir);
    # dists/$dcg/binary-$arch/Packages
    executeOrDie("dpkg-scanpackages -a ".$dcgArchitecture." pool/".$archiveDCG." /dev/null >".$binArchDir."/Packages");
    zipAndChmod($binArchDir."/Packages");
    ## dists/$dcg/Contents-$arch
    #chdir $dcgDistsDCGDir || cleanupAndDie("FATAL: Cannot chdir to ".$dcgDistsDCGDir."\n");
    #executeOrDie("apt-ftparchive contents main/binary-".$dcgArchitecture." /dev/null >Contents-".$dcgArchitecture);
    #zipAndChmod("Contents-".$dcgArchitecture);
    # dists/$dcg/binary-$arch/Release
    chdir $dcgFamilyBaseDir."/".$binArchDir || cleanupAndDie("FATAL: Cannot chdir to ".$dcgFamilyBaseDir."/".$binArchDir."\n");
    open (RELEASE, ">Release") || cleanupAndDie("FATAL: Cannot write to ".pwd()."/Release\n");
    print RELEASE "Archive: ".$archiveDCG."\n";
    print RELEASE "Version: ".$dcgGeneration."\n";
    print RELEASE "Component: main\n";
    print RELEASE "Origin: ".$main::repositoryLongNames{$main::targetRepository}."\n";
    print RELEASE "Label: ".$main::repositoryLongNames{$main::targetRepository}."\n";
    print RELEASE "Architecture: ".$dcgArchitecture."\n";
    close RELEASE;
    chmod 0755, "Release";
    #zipAndChmod("Release");

  }

  # dists/$dcg/Release
 
  chdir $dcgDistsDCGDir || cleanupAndDie("FATAL: Cannot chdir to ".$dcgDistsDCGDir."\n");
  unlink "Release" || cleanupAndDie("FATAL: Cannot delete old ".pwd()."/Release\n");
  open (RELEASE, ">Release.new") || cleanupAndDie("FATAL: Cannot write to ".pwd()."/Release.new\n");
  print RELEASE "Origin: ".$main::repositoryLongNames{$main::targetRepository}."\n";
  print RELEASE "Label: ".$main::repositoryLongNames{$main::targetRepository}."\n";
  print RELEASE "Suite: ".$archiveDCG."\n";
  print RELEASE "Codename: ".$archiveDCG."\n";
  print RELEASE "Version: ".$dcgGeneration."\n";
  print RELEASE "Architectures: ".$main::distributionArchitectures{$dcgDistribution}."\n";
  print RELEASE "Components: main\n";
  print RELEASE "Description: Auto-generated by pba-archiver\n";
  #print RELEASE "Date: ".time2str("%a, %e %h %Y %H:%M:%S %z",time())."\n"; # RFC822 format, like date -R
  close RELEASE;
  executeOrDie("apt-ftparchive release . 2>/dev/null >>Release.new");
  rename "Release.new", "Release" || cleanupAndDie("FATAL: Cannot rename ".pwd()."/Release.new to Release\n");
  signAndChmod("Release");

  # sources.list.d/$distribution/$dcg.list
  my $dcgSourcesListDir = $repositoryRoot."/".$main::distributionFamilies{$dcgDistribution}."/sources.list.d/".$dcgDistribution;
  checkDirectory($dcgSourcesListDir);
  chdir ($dcgSourcesListDir) || cleanupAndDie("FATAL: Cannot chdir to ".$dcgSourcesListDir."\n");
  open (LIST, ">".$archiveDCG.".list") || cleanupAndDie("FATAL: Cannot write ".$dcgSourcesListDir."/".$archiveDCG.".list\n");
  print LIST "deb ".$main::repositoryBaseURIs{$main::targetRepository}."/".$main::distributionFamilies{$dcgDistribution}." ".$archiveDCG." main\n"; 
  print LIST "# deb-src ".$main::repositoryBaseURIs{$main::targetRepository}."/".$main::distributionFamilies{$dcgDistribution}." ".$archiveDCG." main\n"; 
  close LIST;
  chmod 0755, $archiveDCG.".list";

  # suggest cleanup
  if (scalar(@obsoleteDebs) > 0) {
    if ($main::eXpungeOldDebs) {
      print "Removed the following obsolete .deb files in ".$dcgPoolDCGDir.":\n";
      foreach my $obsoleteDeb (@obsoleteDebs) {
        print $obsoleteDeb."\n";
      }
    } else {
      print "Suggesting to remove some obsolete .deb files:\n/bin/rm -fv";
      foreach my $obsoleteDeb (@obsoleteDebs) {
        print " \\\n  ".$dcgPoolDCGDir."/".$obsoleteDeb;
      }
    }
    print "\n";
  } else {
    print "No obsolete .deb files to remove.\n";
  }
  print "\n";

}



1;
