#!/usr/bin/perl
#
# $Id$
#
# (C) ITEG IT-Engineers GmbH 2016
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

use strict;


require "/usr/lib/pba/pba-functions.pl";
require "/usr/lib/pba/archive-functions.pl";


sub scanForIncomingTargzFiles() {
  @main::packageFilesToImport = ();
  my @dirsToScan = split / +/, $main::incomingDirs;
  foreach my $dirToScan (@dirsToScan) {
    ($main::verbosity > 2) && print "Scanning incoming directory ".$dirToScan."\n";
    chdir $dirToScan || cleanupAndDie("FATAL: Cannot chdir to ".$dirToScan."\n");
    my @files = glob("*.rpms.tar.gz");
    foreach my $file (@files) {
      next if (! -f $file); # skip dirs
      ($main::verbosity > 2) && print "Found tarball: ".$file."\n";
      push @main::packageFilesToImport, $dirToScan."/".$file;
    }
  }
}


sub initRPMArchiveFunctions() {
  initArchiveFunctions();

  $main::configParams{"TargetRpmRepository"} = "";
}


sub checkRPMArchiverConfig() {
  checkArchiverConfig();

  $main::targetRpmRepository = $main::configParams{"TargetRpmRepository"};
  ($main::targetRpmRepository eq "") && cleanupAndDie("FATAL: No TargetRpmRepository specified in ".$main::configFile."\n");
  if ($main::verbosity > 2) {
    print "Target RPM repository: ".$main::targetRpmRepository."\n";
  }

  my $repositoryRoot = $main::repositoryRoots{$main::targetRpmRepository};
  ($repositoryRoot eq "") && cleanupAndDie("FATAL: Don't know where my RPM repository root is.\n");
  (not -d $repositoryRoot) && cleanupAndDie("FATAL: RPM repository root directory does not exist: ".$repositoryRoot."\n");
  ($main::verbosity > 2) && print "RPM repository root looks good: ".$repositoryRoot."\n";
}


sub importRpmFile( $ ) {
  # $1 = fullpath of file to import
  # returns distribution-category-generation
  my $rpmFile = shift;
  ($rpmFile eq "") && cleanupAndDie("FATAL: importRpmFile(\$) called w/o argument.\n");
  (-f $rpmFile) || cleanupAndDie("FATAL: importRpmFile(".$rpmFile."): No such file.\n");

  ($main::verbosity > 2) && print "Analysing package ".$rpmFile."\n";

  # detect parts from path
  my @pathParts = split("/", $rpmFile);
  my $nrPathParts = scalar(@pathParts);
  my $rpmBasename     = @pathParts[$nrPathParts-1];
  my $rpmArch         = @pathParts[$nrPathParts-2];
  my $rpmDistribution = @pathParts[$nrPathParts-3];
  my $rpmCatGen       = @pathParts[$nrPathParts-4];
  my ($rpmCategory, $rpmGeneration) = split(/-/, $rpmCatGen, 2);
   
  my $rpmInternalName=`rpm -q --queryformat '%{NAME}' -p $rpmFile`;
  my $rpmInternalArch=`rpm -q --queryformat '%{ARCH}' -p $rpmFile`;

  if ($main::verbosity > 3) {
    print "RPM's Arch: ".$rpmArch."\n";
    print "RPM's Distro: ".$rpmDistribution."\n";
    print "RPM's Category: ".$rpmCategory."\n";
    print "RPM's Generation: ".$rpmGeneration."\n";
    print "RPM's Internal name: ".$rpmInternalName."\n";
    print "RPM's Internal arch: ".$rpmInternalArch."\n";
  }

  my $repositorySubDir = $main::repositoryRoots{$main::targetRpmRepository}."/".$rpmCategory."-".$rpmGeneration."/".$rpmDistribution."/".$rpmArch;
  if (! -d $repositorySubDir) {
    checkDirectory($repositorySubDir);
  }
  chdir $repositorySubDir || cleanupAndDie("FATAL: Cannot chdir to ".$repositorySubDir."\n");

  # rm outdated RPMs
  #($main::verbosity > 2) && print "In ".pwd()." glob-ing for ".$rpmInternalName."-[0-9]*.".$rpmInternalArch.".rpm\n";
  my @oldRpmFiles = glob($rpmInternalName."-[0-9]*.".$rpmInternalArch.".rpm");
  if (scalar(@oldRpmFiles) > 0) {
    if ($main::eXpungeOldDebs) {
      print "Removing obsolete .rpm files in ".$repositorySubDir."\n";
      foreach my $oldRpmFile (@oldRpmFiles) {
        unlink $oldRpmFile || cleanupAndDie("FATAL: Cannot delete ".pwd()."/".$oldRpmFile."\n");
      }
    }
  } else {
    ($main::verbosity > 1) && print "No old rpm files matching ".$rpmInternalName."-[0-9]*.".$rpmInternalArch.".rpm\n";
  }

  # mv in new one
  #executeOrDie("/bin/cp -av ".$rpmFile." .");
  executeOrDie("/bin/cp -v --preserve=timestamp -d ".$rpmFile." .");

  return $rpmDistribution."-".$rpmCategory."-".$rpmGeneration;
}


sub refreshRPMArchive ( $ ) { 
  # $1 = distribution-category-generation
  my $archiveDCG = shift;
  ($archiveDCG eq "") && cleanupAndDie("FATAL: refreshDCGArchive(\$) called w/o argument.\n");

  my ($dcgDistribution, $dcgCategory, $dcgGeneration) = split /-/, $archiveDCG, 3;
  (defined $main::distributionLongNames{$dcgDistribution}) || cleanupAndDie("FATAL: Unknown distribution ".$dcgDistribution."\n");
  my $repositoryRoot = $main::repositoryRoots{$main::targetRpmRepository}."/".$dcgCategory."-".$dcgGeneration;
  chdir $repositoryRoot || cleanupAndDie("FATAL: Cannot chdir to ".$repositoryRoot."\n");

  #($main::verbosity > 2) && print "Refreshing Release info for ".$archiveDCG."\n";

  # check / autocreate repometadatafile
  my $repoFile = $repositoryRoot."/".$dcgCategory."-".$dcgGeneration.".repo";
  if (! -f $repoFile) {
    ($main::verbosity > 1) && print "Auto-creating ".$repoFile."\n";
    my $repoText = "[".$dcgCategory."-".$dcgGeneration."]\n";
    $repoText .= "name=".$dcgCategory."-".$dcgGeneration."\n";
    #if ($dcgDistribution =~ m/centos[0-9]+/) {
    #  $repoText .= "baseurl=".$main::repositoryBaseURIs{$main::targetRpmRepository}."/".$dcgCategory."-".$dcgGeneration.'/centos$releasever/$basearch/'."\n";
    #  $releasever often includes a target class, like '7Workstation', '7Server', so we had to drop it in favour of more static centos7
    #} else {
      $repoText .= "baseurl=".$main::repositoryBaseURIs{$main::targetRpmRepository}."/".$dcgCategory."-".$dcgGeneration."/".$dcgDistribution.'/$basearch/'."\n";
    #}
    if (length($main::repositoryGPGKeyName{$main::targetRpmRepository}) > 0) {
      $repoText .= "gpgcheck=1\n";
      $repoText .= "gpgkey=file:///etc/pki/rpm-gpg/".$main::repositoryGPGKeyName{$main::targetRpmRepository}."\n";
    } else {
      $repoText .= "gpgcheck=0\n";
      $repoText .= "#gpgkey=file:///etc/pki/rpm-gpg/foobar\n";
    }
    $repoText .= "enabled=1\n";
    printToFile($repoFile, $repoText);
  }

  # update repo
  my $basearch="x86_64";
  my $createRepoCmd = "createrepo";
  if (! -d $repositoryRoot."/".$dcgDistribution."/".$main::targetArchitecture."/cachedir" ) {
    checkDirectory($repositoryRoot."/".$dcgDistribution."/".$main::targetArchitecture."/cachedir");
  }
  $createRepoCmd .= " --cachedir ".$repositoryRoot."/".$dcgDistribution."/".$main::targetArchitecture."/cachedir";
  $createRepoCmd .= " --pretty";
  #$createRepoCmd .= " --database"; # default anyway
  $createRepoCmd .= " ".$repositoryRoot."/".$dcgDistribution."/".$main::targetArchitecture;
  executeOrDie($createRepoCmd);

  print "\n";

}



1;
